@@ -25,7 +25,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_dag24.c 0 2006-12-14 21:13:09Z spa1 $
+ * $Id: format_dag24.c 1255 2007-09-04 22:18:58Z spa1 $
  *
  */
 #define _GNU_SOURCE
@@ -136,7 +136,7 @@ static int dag_init_input(libtrace_t *libtrace) {
 static int dag_config_input(libtrace_t *libtrace, trace_option_t option,
                                 void *data) {
         switch(option) {
-                case TRACE_META_FREQ:
+                case TRACE_OPTION_META_FREQ:
                         DUCK.duck_freq = *(int *)data;
                         return 0;
                 case TRACE_OPTION_SNAPLEN:
@@ -147,13 +147,10 @@ static int dag_config_input(libtrace_t *libtrace, trace_option_t option,
                         return -1;
                 case TRACE_OPTION_FILTER:
                         return -1;
-                default:
-                        trace_set_err(libtrace, TRACE_ERR_UNKNOWN_OPTION,
-                                        "Unknown or unsupported option: %i",
-                                        option);
-                        return -1;
+		case TRACE_OPTION_EVENT_REALTIME:
+			return -1;
         }
-        assert (0);
+	return -1;
 }
 
 static int dag_start_input(libtrace_t *libtrace) {	
@@ -224,7 +221,7 @@ static int dag_get_duckinfo(libtrace_t *libtrace,
 }
 
 
-dag_record_t *dag_get_record(libtrace_t *libtrace) {
+static dag_record_t *dag_get_record(libtrace_t *libtrace) {
         dag_record_t *erfptr = NULL;
         uint16_t size;
 	erfptr = (dag_record_t *) ((char *)FORMAT_DATA->buf + 
@@ -239,7 +236,7 @@ dag_record_t *dag_get_record(libtrace_t *libtrace) {
         return erfptr;
 }
 
-void dag_form_packet(dag_record_t *erfptr, libtrace_packet_t *packet) {
+static void dag_form_packet(dag_record_t *erfptr, libtrace_packet_t *packet) {
         packet->buffer = erfptr;
         packet->header = erfptr;
         packet->type = TRACE_RT_DATA_ERF;
@@ -326,7 +323,7 @@ static libtrace_eventobj_t trace_event_dag(libtrace_t *trace,
 }
 
 static void dag_help(void) {
-        printf("dag format module: $Revision: 1110 $\n");
+        printf("dag format module: $Revision: 1255 $\n");
         printf("Supported input URIs:\n");
         printf("\tdag:/dev/dagn\n");
         printf("\n");
@@ -339,7 +336,7 @@ static void dag_help(void) {
 
 static struct libtrace_format_t dag = {
         "dag",
-        "$Id: format_dag24.c 0 2006-12-14 21:13:09Z spa1 $",
+        "$Id: format_dag24.c 1255 2007-09-04 22:18:58Z spa1 $",
         TRACE_FORMAT_ERF,
         dag_init_input,                 /* init_input */
         dag_config_input,               /* config_input */
