@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_pcap.c 1131 2007-01-26 02:31:48Z perry $
+ * $Id: format_pcap.c 1247 2007-08-01 04:58:15Z spa1 $
  *
  */
 
@@ -129,11 +129,10 @@ static int pcap_config_input(libtrace_t *libtrace,
 			 */
 		case TRACE_OPTION_PROMISC:
 			/* can't do promisc on a trace! fall thru */
-		case TRACE_META_FREQ:
+		case TRACE_OPTION_META_FREQ:
 			/* No meta data for this format */
+		case TRACE_OPTION_EVENT_REALTIME:
 		default:
-			trace_set_err(libtrace,TRACE_ERR_UNKNOWN_OPTION,
-					"Unknown option %i", option);
 			return -1;
 	}
 	assert(0);
@@ -188,11 +187,15 @@ static int pcapint_config_input(libtrace_t *libtrace,
 		case TRACE_OPTION_PROMISC:
 			DATA(libtrace)->promisc=*(int*)data;
 			return 0;
-		case TRACE_META_FREQ:
+		case TRACE_OPTION_META_FREQ:
 			/* No meta-data for this format */
+		case TRACE_OPTION_EVENT_REALTIME:
+			/* live interface is always real-time! */
 		default:
-			trace_set_err(libtrace,TRACE_ERR_UNKNOWN_OPTION,
-					"Unknown option %i", option);
+			/* Don't set an error here - trace_config will try
+			 * to handle the option when we return. If it can't
+			 * deal with it, then it will do the necessary
+			 * error-setting. */
 			return -1;
 	}
 	assert(0);
@@ -263,7 +266,7 @@ static int pcap_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 
 	assert(libtrace->format_data);
 	linktype = pcap_datalink(DATA(libtrace)->input.pcap);
-	packet->type = pcap_dlt_to_rt(linktype);
+	packet->type = pcap_linktype_to_rt(linktype);
 
 	packet->buf_control = TRACE_CTRL_PACKET;
 
@@ -313,7 +316,7 @@ static int pcap_write_packet(libtrace_out_t *libtrace,
 	/* If this packet cannot be converted to a pcap linktype then
 	 * pop off the top header until it can be converted
 	 */
-	while (libtrace_to_pcap_dlt(trace_get_link_type(packet))==~0U) {
+	while (libtrace_to_pcap_linktype(trace_get_link_type(packet))==~0U) {
 		if (!demote_packet(packet)) {
 			trace_set_err_out(libtrace, 
 				TRACE_ERR_NO_CONVERSION,
@@ -322,12 +325,13 @@ static int pcap_write_packet(libtrace_out_t *libtrace,
 		}
 	}
 
+
 	if (!OUTPUT.trace.pcap) {
-		OUTPUT.trace.pcap = pcap_open_dead(
-			libtrace_to_pcap_dlt(trace_get_link_type(packet)),
-			65536);
+		int linktype=libtrace_to_pcap_dlt(trace_get_link_type(packet));
+		OUTPUT.trace.pcap = pcap_open_dead(linktype,65536);
 		if (!OUTPUT.trace.pcap) {
-			trace_set_err_out(libtrace,TRACE_ERR_INIT_FAILED,"Failed to open dead trace: %s\n",
+			trace_set_err_out(libtrace,TRACE_ERR_INIT_FAILED,
+					"Failed to open dead trace: %s\n",
 					pcap_geterr(OUTPUT.trace.pcap));
 		}
 		OUTPUT.trace.dump = pcap_dump_open(OUTPUT.trace.pcap,
@@ -408,11 +412,11 @@ static int pcapint_write_packet(libtrace_out_t *libtrace,
 }
 
 static libtrace_linktype_t pcap_get_link_type(const libtrace_packet_t *packet) {
-	/* pcap doesn't store dlt in the framing header so we need
+	/* pcap doesn't store linktype in the framing header so we need
 	 * rt to do it for us 
 	 */
-	int linktype = rt_to_pcap_dlt(packet->type);
-	return pcap_dlt_to_libtrace(linktype);
+	int linktype = rt_to_pcap_linktype(packet->type);
+	return pcap_linktype_to_libtrace(linktype);
 }
 
 static libtrace_direction_t pcap_set_direction(libtrace_packet_t *packet,
@@ -510,10 +514,20 @@ static int pcap_get_capture_length(const libtrace_packet_t *packet) {
 static int pcap_get_wire_length(const libtrace_packet_t *packet) {
 	struct pcap_pkthdr *pcapptr = 0;
 	pcapptr = (struct pcap_pkthdr *)packet->header;
-	if (packet->type==pcap_dlt_to_rt(TRACE_DLT_EN10MB))
+	if (packet->type==pcap_linktype_to_rt(TRACE_DLT_EN10MB))
 		return pcapptr->len+4; /* Include the missing FCS */
-	else
-		return pcapptr->len;
+	else if (packet->type==pcap_linktype_to_rt(TRACE_DLT_IEEE802_11_RADIO)) {
+		/* If the packet is Radiotap and the flags field indicates
+		 * that the FCS is not included in the 802.11 frame, then
+		 * we need to add 4 to the wire-length to account for it.
+		 */
+		uint8_t flags;
+		trace_get_wireless_flags(trace_get_link(packet), 
+				trace_get_link_type(packet), &flags);
+		if ((flags & TRACE_RADIOTAP_F_FCS) == 0)
+			return pcapptr->len + 4;
+	}
+	return pcapptr->len;
 }
 
 static int pcap_get_framing_length(UNUSED const libtrace_packet_t *packet) {
@@ -539,7 +553,7 @@ static int pcap_get_fd(const libtrace_t *trace) {
 }
 
 static void pcap_help(void) {
-	printf("pcap format module: $Revision: 1131 $\n");
+	printf("pcap format module: $Revision: 1247 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcap:/path/to/file\n");
 	printf("\n");
@@ -551,7 +565,7 @@ static void pcap_help(void) {
 }
 
 static void pcapint_help(void) {
-	printf("pcapint format module: $Revision: 1131 $\n");
+	printf("pcapint format module: $Revision: 1247 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcapint:interface\n");
 	printf("\n");
@@ -565,7 +579,7 @@ static void pcapint_help(void) {
 
 static struct libtrace_format_t pcap = {
 	"pcap",
-	"$Id: format_pcap.c 1131 2007-01-26 02:31:48Z perry $",
+	"$Id: format_pcap.c 1247 2007-08-01 04:58:15Z spa1 $",
 	TRACE_FORMAT_PCAP,
 	pcap_init_input,		/* init_input */
 	pcap_config_input,		/* config_input */
@@ -600,7 +614,7 @@ static struct libtrace_format_t pcap = {
 
 static struct libtrace_format_t pcapint = {
 	"pcapint",
-	"$Id: format_pcap.c 1131 2007-01-26 02:31:48Z perry $",
+	"$Id: format_pcap.c 1247 2007-08-01 04:58:15Z spa1 $",
 	TRACE_FORMAT_PCAP,
 	pcapint_init_input,		/* init_input */
 	pcapint_config_input,		/* config_input */
