@@ -25,7 +25,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_rt.c 1131 2007-01-26 02:31:48Z perry $
+ * $Id: format_rt.c 1263 2007-09-05 04:34:10Z perry $
  *
  */
 
@@ -87,7 +87,6 @@ struct rt_format_data_t {
 	libtrace_t *dummy_duck;
 	libtrace_t *dummy_erf;
 	libtrace_t *dummy_pcap;
-	libtrace_t *dummy_wag;
 	libtrace_t *dummy_linux;
 };
 
@@ -197,7 +196,6 @@ static int rt_init_input(libtrace_t *libtrace) {
 	RT_INFO->dummy_duck = NULL;
 	RT_INFO->dummy_erf = NULL;
 	RT_INFO->dummy_pcap = NULL;
-	RT_INFO->dummy_wag = NULL;
 	RT_INFO->dummy_linux = NULL;
 	RT_INFO->pkt_buffer = NULL;
 	RT_INFO->buf_current = NULL;
@@ -247,11 +245,6 @@ static int rt_start_input(libtrace_t *libtrace) {
 }
 
 static int rt_pause_input(libtrace_t *libtrace) {
-	close(RT_INFO->input_fd);
-	return 0;
-}
-
-static int rt_fin_input(libtrace_t *libtrace) {
         rt_header_t close_msg;
 
 	close_msg.type = TRACE_RT_CLOSE;
@@ -264,6 +257,12 @@ static int rt_fin_input(libtrace_t *libtrace) {
 		printf("Failed to send close message to server\n");
 	
 	}
+
+	close(RT_INFO->input_fd);
+	return 0;
+}
+
+static int rt_fin_input(libtrace_t *libtrace) {
 	if (RT_INFO->dummy_duck)
 		trace_destroy_dead(RT_INFO->dummy_duck);
 
@@ -273,13 +272,8 @@ static int rt_fin_input(libtrace_t *libtrace) {
 	if (RT_INFO->dummy_pcap)
 		trace_destroy_dead(RT_INFO->dummy_pcap);
 
-	if (RT_INFO->dummy_wag)
-		trace_destroy_dead(RT_INFO->dummy_wag);
-
 	if (RT_INFO->dummy_linux)
 		trace_destroy_dead(RT_INFO->dummy_linux);
-
-	close(RT_INFO->input_fd);
 	free(libtrace->format_data);
         return 0;
 }
@@ -391,12 +385,6 @@ static int rt_set_format(libtrace_t *libtrace, libtrace_packet_t *packet)
 			}
 			packet->trace = RT_INFO->dummy_erf;
 			break;
-		case TRACE_RT_DATA_WAG:
-			if (!RT_INFO->dummy_wag) {
-				RT_INFO->dummy_wag = trace_create_dead("wtf:-");
-			}
-			packet->trace = RT_INFO->dummy_wag;
-			break;
 		case TRACE_RT_DATA_LINUX_NATIVE:
 			if (!RT_INFO->dummy_linux) {
 				RT_INFO->dummy_linux = trace_create_dead("int:");
@@ -542,6 +530,7 @@ static int rt_read_packet_versatile(libtrace_t *libtrace,
 	} else {
 		switch(packet->type) {
 			case TRACE_RT_STATUS:
+			case TRACE_RT_METADATA:
 				if (rt_read(libtrace, &packet->buffer, 
 					(size_t)RT_INFO->rt_hdr.length,
 					blocking) != 
@@ -577,6 +566,10 @@ static int rt_read_packet_versatile(libtrace_t *libtrace,
 				break;
 			case TRACE_RT_LOSTCONN:
 				break;
+			case TRACE_RT_SERVERSTART:
+				break;
+			case TRACE_RT_CLIENTDROP:
+				break;
 			default:
 				printf("Bad rt type for client receipt: %d\n",
 					packet->type);
@@ -595,6 +588,7 @@ static int rt_read_packet(libtrace_t *libtrace,
 
 
 static int rt_get_capture_length(const libtrace_packet_t *packet) {
+	rt_metadata_t *rt_md_hdr;
 	switch (packet->type) {
 		case TRACE_RT_STATUS:
 			return sizeof(rt_status_t);
@@ -620,6 +614,15 @@ static int rt_get_capture_length(const libtrace_packet_t *packet) {
 			return 0;
 		case TRACE_RT_LOSTCONN:
 			return 0;
+		case TRACE_RT_SERVERSTART:
+			return 0;
+		case TRACE_RT_CLIENTDROP:
+			return 0;
+		case TRACE_RT_METADATA:
+			/* This is a little trickier to work out */
+			rt_md_hdr = (rt_metadata_t *)packet->buffer;
+			return rt_md_hdr->label_len + rt_md_hdr->value_len + 
+				sizeof(rt_metadata_t);
 		default:
 			printf("Unknown type: %d\n", packet->type);
 			
@@ -692,7 +695,7 @@ static void rt_help(void) {
 
 static struct libtrace_format_t rt = {
         "rt",
-        "$Id: format_rt.c 1131 2007-01-26 02:31:48Z perry $",
+        "$Id: format_rt.c 1263 2007-09-05 04:34:10Z perry $",
         TRACE_FORMAT_RT,
         rt_init_input,            	/* init_input */
         NULL,                           /* config_input */
