@@ -128,8 +128,11 @@
 
 /** API version as 2 byte hex digits, eg 0xXXYYZZ */
 #define LIBTRACE_API_VERSION \
-            ((3<<16)|(0<<8)|(0))
+            ((3<<16)|(0<<8)|(3))
 
+#define LIBTRACE_SVN_REVISION 0
+#define DAG_DRIVER_V ""
+    
 #ifdef __cplusplus 
 extern "C" { 
 #endif
@@ -199,15 +202,20 @@ enum {
 	TRACE_ERR_UNSUPPORTED	= -7
 };
 
-/** Enumeration of DLT types supported by libtrace 
- * These aren't actually DLTs, but LINKTYPE_'s, so becareful when modifying 
- * this.
- */ 
+/** Enumeration of DLT supported by libtrace 
+ */
 typedef enum {
 	TRACE_DLT_NULL = 0,
 	TRACE_DLT_EN10MB = 1,
+	TRACE_DLT_PPP = 9,
 	TRACE_DLT_ATM_RFC1483 = 11,
-	TRACE_DLT_RAW = 101,
+	/* Sigh. This is handled in files with LINKTYPE's */
+#ifdef __OpenBSD__
+	TRACE_DLT_RAW = 14,	
+#else
+	TRACE_DLT_RAW = 12,
+#endif
+	TRACE_DLT_LINKTYPE_RAW = 101,
 	TRACE_DLT_IEEE802_11 = 105,
 	TRACE_DLT_LINUX_SLL = 113,
 	TRACE_DLT_PFLOG = 117,
@@ -232,7 +240,9 @@ typedef enum {
        TRACE_TYPE_AAL5 = 13,
        TRACE_TYPE_DUCK = 14,	     /**< Pseudo link layer for DUCK packets */
        TRACE_TYPE_80211_RADIO = 15,  /**< Radiotap + 802.11 */
-       TRACE_TYPE_LLCSNAP = 16       /**< Raw LLC/SNAP */
+       TRACE_TYPE_LLCSNAP = 16,      /**< Raw LLC/SNAP */
+       TRACE_TYPE_PPP = 17,	     /**< PPP frames */
+       TRACE_TYPE_METADATA = 18	     	/**< WDCAP-style meta-data */
 	
 } libtrace_linktype_t;
 
@@ -250,7 +260,10 @@ enum base_format_t {
 	TRACE_FORMAT_LEGACY_ETH   =8,
 	TRACE_FORMAT_LINUX_NATIVE =9,
 	TRACE_FORMAT_DUCK	  =10,
-	TRACE_FORMAT_BPF	  =11
+	TRACE_FORMAT_BPF	  =11,
+	TRACE_FORMAT_TSH	  =12,
+	TRACE_FORMAT_ATMHDR	  =13,
+	TRACE_FORMAT_LEGACY_NZIX  =14
 };
 
 /* RT protocol packet types */
@@ -272,6 +285,9 @@ typedef enum {
 	TRACE_RT_DUCK_2_4	=13,/**< Dag 2.4 Duck */
 	TRACE_RT_DUCK_2_5 	=14,/**< Dag 2.5 Duck */
 	TRACE_RT_LOSTCONN 	=15,/**< Lost connection to server */
+	TRACE_RT_SERVERSTART	=16,/**< Reliable server has been restarted */
+	TRACE_RT_CLIENTDROP	=17,/**< Reliable client was lost */
+	TRACE_RT_METADATA	=18,/**< Packet contains server meta-data */
 
 	TRACE_RT_DATA_SIMPLE	= 1000, /**< Trace types that know their link
 					  * type
@@ -282,7 +298,10 @@ typedef enum {
 	TRACE_RT_DATA_LEGACY_POS=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_LEGACY_POS, 
 	TRACE_RT_DATA_LEGACY_ETH=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_LEGACY_ETH, 
 	TRACE_RT_DATA_LINUX_NATIVE=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_LINUX_NATIVE,
+	TRACE_RT_DATA_TSH	=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_TSH,
 
+	TRACE_RT_DATA_ATMHDR = TRACE_RT_DATA_SIMPLE + TRACE_FORMAT_ATMHDR,
+	TRACE_RT_DATA_LEGACY_NZIX=TRACE_RT_DATA_SIMPLE + TRACE_FORMAT_LEGACY_NZIX,
 	TRACE_RT_DATA_DLT		= 2000, /**< Pcap doesn't store the
 						  * linktype per packet, and
 						  * thus we have to store it
@@ -307,6 +326,9 @@ typedef struct libtrace_packet_t {
 	void *buffer;			/**< allocated buffer */
 	libtrace_rt_types_t  type; 	/**< rt protocol type for the packet */
 	buf_control_t buf_control; 	/**< who owns the memory */
+	int capture_length;		/**< Cached capture length */
+	void *l3_header;		/**< Cached l3 header */
+	uint16_t l3_ethertype;		/**< Cached l3 ethertype */
 } libtrace_packet_t;
 
 
@@ -316,8 +338,8 @@ typedef struct libtrace_packet_t {
  * convention.
  */
 typedef enum {
-	TRACE_DIR_OUTGOING = 0,		/**< Packets originating "outside" */
-	TRACE_DIR_INCOMING = 1,		/**< Packets originating "inside" */
+	TRACE_DIR_OUTGOING = 0,		/**< Packets originating "inside" */
+	TRACE_DIR_INCOMING = 1,		/**< Packets originating "outside" */
 	TRACE_DIR_OTHER	   = 2		/**< Packets with an unknown direction, or one that's unknown */
 } libtrace_direction_t;
 
@@ -337,7 +359,10 @@ typedef enum {
     TRACE_RADIOTAP_ANTENNA = 11, /**< Antenna frame was rx'd or tx'd on (uint8) */
     TRACE_RADIOTAP_DB_ANTSIGNAL = 12, /**< Signal power in dB from a fixed reference (uint8) */
     TRACE_RADIOTAP_DB_ANTNOISE = 13, /**< Noise power in dB from a fixed reference (uint8) */
-    TRACE_RADIOTAP_FCS = 14, /**< Received frame check sequence (uint32) */
+    TRACE_RADIOTAP_RX_FLAGS = 14, /** Properties of received frame (uint16) */
+    TRACE_RADIOTAP_TX_FLAGS = 15, /** Properties of transmitted frame (uint16) */
+    TRACE_RADIOTAP_RTS_RETRIES = 16, /** Number of rts retries frame used (uint8) */
+    TRACE_RADIOTAP_DATA_RETRIES = 17, /** Number of unicast retries a transmitted frame used (uint8) */
     TRACE_RADIOTAP_EXT = 31
 } libtrace_radiotap_field_t;
 
@@ -526,7 +551,7 @@ typedef struct libtrace_atm_nni_cell
 /** Captured UNI cell.
  *
  * Endance don't capture the HEC, presumably to keep alignment.  This 
- * version of the \ref{libtrace_atm_cell} is used when dealing with dag
+ * version of the \ref libtrace_atm_cell  is used when dealing with dag
  * captures of uni cells.
  *
  */
@@ -542,7 +567,7 @@ typedef struct libtrace_atm_capture_cell
 /** Captured NNI cell.
  *
  * Endance don't capture the HEC, presumably to keep alignment.  This 
- * version of the \ref{libtrace_atm_nni_cell} is used when dealing with dag
+ * version of the \ref libtrace_atm_nni_cell  is used when dealing with dag
  * captures of nni cells.
  *
  */
@@ -555,12 +580,13 @@ typedef struct libtrace_atm_nni_capture_cell
   LT_BITFIELD32 hec:8;		/**< Header Error Control */
 } PACKED libtrace_atm_nni_capture_cell_t;
 
-/** POS header */
-typedef struct libtrace_pos
+/** PPP header */
+typedef struct libtrace_pps
 {
- uint16_t header;
- uint16_t ether_type;		/**< Ether Type */
-} PACKED libtrace_pos_t;
+ uint8_t addres;		/**< PPP Address (0xFF - All stations) */
+ uint8_t header;		/**< PPP Control (0x03 - Unnumbered info) */
+ uint16_t protocol;		/**< PPP Protocol (htons(0x0021) - IPv4 */
+} PACKED libtrace_ppp_t;
 
 /** 802.11 header */
 typedef struct libtrace_80211_t {
@@ -652,9 +678,6 @@ DLLEXPORT void trace_help(void);
  *  - rt:hostname:port
  *  - rtclient:hostname	(deprecated)
  *  - rtclient:hostname:port (deprecated)
- *  - wag:/dev/wagcard
- *  - wtf:-
- *  - wtf:/path/to/wtf/file
  *
  *  If an error occured when attempting to open the trace file, an error
  *  trace is returned and trace_get_error should be called to find out
@@ -684,7 +707,7 @@ DLLEXPORT libtrace_t *trace_create_dead(const char *uri);
  *  - wtf:/path/to/wtf/file
  *
  *  If an error occured when attempting to open the output trace, NULL is returned 
- *  and trace_errno is set. Use trace_perror() to get more information
+ *  and trace_errno is set. Use trace_perror_output() to get more information
  */
 DLLEXPORT libtrace_out_t *trace_create_output(const char *uri);
 
@@ -718,10 +741,12 @@ DLLEXPORT int trace_start_output(libtrace_out_t *libtrace);
 
 /** Valid trace capture options */
 typedef enum {
-	TRACE_OPTION_SNAPLEN, /**< Number of bytes captured */
-	TRACE_OPTION_PROMISC, /**< Capture packets to other hosts */
-	TRACE_OPTION_FILTER,  /**< Apply this filter to all packets recieved */
-	TRACE_META_FREQ	      /**< Frequency of meta-data information, e.g. DUCK packets */
+	TRACE_OPTION_SNAPLEN, 	/**< Number of bytes captured */
+	TRACE_OPTION_PROMISC, 	/**< Capture packets to other hosts */
+	TRACE_OPTION_FILTER,  	/**< Apply this filter to all packets */
+	TRACE_OPTION_META_FREQ,	/**< Frequency of meta-data information, e.g. DUCK packets */
+	/** trace_event ignores gaps between packets when reading traces off disk */
+	TRACE_OPTION_EVENT_REALTIME
 } trace_option_t;
 
 /** Sets an input config option
@@ -926,10 +951,10 @@ DLLEXPORT int trace_write_packet(libtrace_out_t *trace, libtrace_packet_t *packe
 DLLEXPORT SIMPLE_FUNCTION
 void *trace_get_link(const libtrace_packet_t *packet);
 
-/** get a pointer to the IP header (if any)
+/** get a pointer to the IPv4 header (if any)
  * @param packet  	the packet opaque pointer
  *
- * @return a pointer to the IP header, or NULL if there is no IP header
+ * @return a pointer to the IPv4 header, or NULL if there is no IPv4 header
  */
 DLLEXPORT SIMPLE_FUNCTION
 libtrace_ip_t *trace_get_ip(libtrace_packet_t *packet);
@@ -942,6 +967,20 @@ libtrace_ip_t *trace_get_ip(libtrace_packet_t *packet);
 DLLEXPORT SIMPLE_FUNCTION
 libtrace_ip6_t *trace_get_ip6(libtrace_packet_t *packet);
 
+/** Get a pointer to the layer 3 header.
+ * @param packet  		The packet opaque pointer
+ * @param[out] ethertype	The ethertype of the layer 3 header
+ * @param[out] remaining	The amount of space available after this header
+ * 				has been removed.
+ *
+ * @return a pointer to the layer 3 header.
+ * remaining may be NULL, otherwise it will be filled in by the remaining size
+ * of the captured packet.
+ */
+DLLEXPORT SIMPLE_FUNCTION
+void *trace_get_layer3(libtrace_packet_t *packet,
+		uint16_t *ethertype, uint32_t *remaining);
+
 /** Gets a pointer to the transport layer header (if any)
  * @param packet        a pointer to a libtrace_packet structure
  * @param[out] proto	transport layer protocol
@@ -1036,14 +1075,14 @@ DLLEXPORT void *trace_get_vlan_payload_from_ethernet_payload(
  *
  * @return a pointer to the tcp payload, or NULL if the payload isn't present.
  *
- * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the TCP header (and any TCP options)
- * have been removed.
+ * Remaining may be NULL.  If remaining is not NULL it must point to the number
+ * of bytes captured of the TCP header and beyond.  It will be updated after
+ * this function to the number of bytes remaining after the TCP header (and any
+ * TCP options) have been removed.
  *
- * @note This is similar to trace_get_transport_from_ip in libtrace2
  */
-DLLEXPORT void *trace_get_payload_from_tcp(libtrace_tcp_t *tcp, uint32_t *remaining);
+DLLEXPORT void *trace_get_payload_from_tcp(libtrace_tcp_t *tcp, 
+		uint32_t *remaining);
 
 /** Gets a pointer to the payload given a pointer to a udp header
  * @param udp           The udp Header
@@ -1052,11 +1091,9 @@ DLLEXPORT void *trace_get_payload_from_tcp(libtrace_tcp_t *tcp, uint32_t *remain
  * @return a pointer to the udp payload, or NULL if the payload isn't present.
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the TCP header (and any TCP options)
- * have been removed.
+ * of bytes captured of the UDP header and beyond.  It will be updated after
+ * this function to the number of bytes remaining after the UDP header.
  *
- * @note This is similar trace_get_transport_from_ip in libtrace2
  */
 DLLEXPORT void *trace_get_payload_from_udp(libtrace_udp_t *udp, uint32_t *remaining);
 
@@ -1066,14 +1103,13 @@ DLLEXPORT void *trace_get_payload_from_udp(libtrace_udp_t *udp, uint32_t *remain
  *
  * @return a pointer to the icmp payload, or NULL if the payload isn't present.
  *
- * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the TCP header (and any TCP options)
- * have been removed.
- *
- * @note This is similar to trace_get_payload_from_icmp in libtrace2
+ * Remaining may be NULL.  If remaining is not NULL it must point to the number
+ * of bytes captured of the ICMP header and beyond.  It will be updated after
+ * this function to the number of bytes remaining after the ICMP header.
+ * 
  */
-DLLEXPORT void *trace_get_payload_from_icmp(libtrace_icmp_t *icmp, uint32_t *remaining);
+DLLEXPORT void *trace_get_payload_from_icmp(libtrace_icmp_t *icmp, 
+		uint32_t *remaining);
 
 /** get a pointer to the TCP header (if any)
  * @param packet  	the packet opaque pointer
@@ -1090,9 +1126,9 @@ libtrace_tcp_t *trace_get_tcp(libtrace_packet_t *packet);
  * @return a pointer to the TCP header, or NULL if this is not a TCP packet
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the TCP header (and any TCP options)
- * have been removed.
+ * of bytes captured of the TCP header and beyond.  It will be updated after
+ * this function to the number of bytes remaining after the TCP header (and any
+ * TCP options) have been removed.
  *
  * @note The last parameter has changed from libtrace2
  */
@@ -1114,9 +1150,9 @@ libtrace_udp_t *trace_get_udp(libtrace_packet_t *packet);
  * @return a pointer to the UDP header, or NULL if this is not an UDP packet
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the TCP header (and any TCP options)
- * have been removed.
+ * of bytes captured of the UDP header and beyond.  It will be updated after
+ * this function to the number of bytes remaining after the UDP header have
+ * been removed.
  *
  * @note Beware the change from libtrace2 from skipped to remaining
  */
@@ -1138,9 +1174,8 @@ libtrace_icmp_t *trace_get_icmp(libtrace_packet_t *packet);
  * @return a pointer to the ICMP header, or NULL if this is not an ICMP packet
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the TCP header (and any TCP options)
- * have been removed.
+ * of bytes captured of the ICMP header and beyond.  It will be updated after
+ * this function to the number of bytes remaining after the ICMP header.
  *
  * @note Beware the change from libtrace2 from skipped to remaining
  */
@@ -1166,7 +1201,8 @@ uint8_t *trace_get_source_mac(libtrace_packet_t *packet);
  * @param packet  	the packet opaque pointer
  * @param addr		a pointer to a sockaddr to store the address in, or NULL to use
  * 			static storage.
- * @return NULL if there is no source address, or a sockaddr holding a v4 or v6 address
+ * @return NULL if there is no source address, or a sockaddr holding a v4 or v6
+ * address
  */
 DLLEXPORT SIMPLE_FUNCTION
 struct sockaddr *trace_get_source_address(const libtrace_packet_t *packet,
@@ -1176,7 +1212,8 @@ struct sockaddr *trace_get_source_address(const libtrace_packet_t *packet,
  * @param packet  	the packet opaque pointer
  * @param addr		a pointer to a sockaddr to store the address in, or NULL to use
  * 			static storage.
- * @return NULL if there is no destination address, or a sockaddr holding a v4 or v6 address
+ * @return NULL if there is no destination address, or a sockaddr holding a v4
+ * or v6 address
  */
 DLLEXPORT SIMPLE_FUNCTION
 struct sockaddr *trace_get_destination_address(const libtrace_packet_t *packet,
@@ -1232,7 +1269,8 @@ struct timeval trace_get_timeval(const libtrace_packet_t *packet);
 /** Get the current time in floating point seconds
  * @param packet  	the packet opaque pointer
  *
- * @return time that this packet was seen in 64bit floating point seconds
+ * @return time that this packet was seen in 64bit floating point seconds from
+ * the unix epoch (1970-01-01 00:00:00 UTC).
  */
 DLLEXPORT SIMPLE_FUNCTION
 double trace_get_seconds(const libtrace_packet_t *packet);
@@ -1288,10 +1326,11 @@ DLLEXPORT int trace_seek_erf_timestamp(libtrace_t *trace, uint64_t ts);
  * @note The return size refers to the network-level payload of the packet and
  * does not include any capture framing headers. For example, an Ethernet 
  * packet with an empty TCP packet will return sizeof(ethernet_header) + 
- * sizeof(ip_header) + sizeof(tcp_header).
+ * sizeof(ip_header) + sizeof(tcp_header), but not the capture file 
+ * (pcap/erf/etc) header.
  */
 DLLEXPORT SIMPLE_FUNCTION
-size_t trace_get_capture_length(const libtrace_packet_t *packet);
+size_t trace_get_capture_length(libtrace_packet_t *packet);
 
 /** Get the size of the packet as it was seen on the wire.
  * @param packet  	the packet opaque pointer
@@ -1299,6 +1338,11 @@ size_t trace_get_capture_length(const libtrace_packet_t *packet);
  * @note Due to the trace being a header capture, or anonymisation this may
  * not be the same as the Capture Len.
  * @note trace_getwire_length \em{includes} FCS.
+ * @note The return size refers to the network-level payload of the packet and
+ * does not include any capture framing headers. For example, an Ethernet 
+ * packet with an empty TCP packet will return sizeof(ethernet_header) + 
+ * sizeof(ip_header) + sizeof(tcp_header), but not the capture file 
+ * (pcap/erf/etc) header.
  */ 
 DLLEXPORT SIMPLE_FUNCTION
 size_t trace_get_wire_length(const libtrace_packet_t *packet);
@@ -1614,15 +1658,6 @@ DLLEXPORT bool trace_get_wireless_tx_power_dbm(void *linkptr, libtrace_linktype_
 DLLEXPORT bool trace_get_wireless_antenna(void *linkptr,
         libtrace_linktype_t linktype, uint8_t *antenna);
 
-/** Get the wireless Frame Check Sequence field
- * @param link the wireless header
- * @param linktype the linktype of the wireless header passed in
- * @param[out] fcs the Frame Check Sequence of the frame. 
- * @return true if the field was available, false if not.
- */
-DLLEXPORT bool trace_get_wireless_fcs(void *linkptr,
-        libtrace_linktype_t linktype, uint32_t *fcs);
-
 /*@}*/
 
 #ifdef __cplusplus
