@@ -1,10 +1,9 @@
 /* This file has the various helper functions used to decode various protocols 
  *
- * $Id: protocols.c 1116 2006-12-20 01:37:51Z spa1 $
+ * $Id: protocols.c 1263 2007-09-05 04:34:10Z perry $
  */ 
 #include "libtrace.h"
 #include "libtrace_int.h"
-#include "wag.h"
 #include <assert.h>
 #include <stdio.h>
 
@@ -21,6 +20,9 @@
 #endif
 
 
+static void *trace_get_payload_from_llcsnap(void *link,
+		uint16_t *type, uint32_t *remaining);
+
 /* Returns the payload from 802.3 ethernet.  Type optionally returned in
  * "type" in host byte order.  This will return a vlan header.
  */
@@ -113,13 +115,9 @@ static void *trace_get_mpls_payload_from_ethernet_payload(void *ethernet,
 
 static void *trace_get_payload_from_80211(void *link, uint16_t *type, uint32_t *remaining)
 {
-	/* TODO: Decode type != DATA
-	 * TODO: We're assuming four address frame here... probably not good
-	 */
-	
 	libtrace_80211_t *wifi;
-	libtrace_802_11_payload_t *eth;
-	uint8_t extra = 0; /* how many QoS bytes to skip */
+	uint16_t *eth; /* ethertype */
+	int8_t extra = 0; /* how many QoS bytes to skip */
 	
 	if (remaining && *remaining < sizeof(libtrace_80211_t))
 		return NULL;
@@ -131,6 +129,11 @@ static void *trace_get_payload_from_80211(void *link, uint16_t *type, uint32_t *
 		return NULL;
 	}
 
+	/* If FromDS and ToDS are both set then we have a four-address
+	 * frame. Otherwise we have a three-address frame */
+	if (!(wifi->to_ds && wifi->from_ds)) 
+		extra -= 6; 
+	
 	/* Indicates QoS field present, see IEEE802.11e-2005 pg 21 */
 	if (wifi->subtype & 0x8) 
 		extra += 2;
@@ -138,11 +141,15 @@ static void *trace_get_payload_from_80211(void *link, uint16_t *type, uint32_t *
 	if (remaining && *remaining < sizeof(*eth))
 		return NULL;
 
-	eth=(libtrace_802_11_payload_t *)((char*)wifi+sizeof(*wifi)+extra);
-
-	if (type) *type=ntohs(eth->type);
-	if (remaining) *remaining = *remaining - sizeof(libtrace_80211_t) - extra - sizeof(*eth);
+	eth=(uint16_t *)((char*)wifi+sizeof(*wifi)+extra);
 	
+	if (*eth == 0xaaaa)
+		/* Payload contains an 802.2 LLC/SNAP frame */
+		return trace_get_payload_from_llcsnap((void *)eth, type, remaining);
+			
+	/* Otherwise we assume an Ethernet II frame */
+	if (type) *type=ntohs(*eth);
+	if (remaining) *remaining = *remaining - sizeof(libtrace_80211_t) - extra - sizeof(*eth);
 	
 	return (void*)((char*)eth+sizeof(*eth));
 }
@@ -205,26 +212,26 @@ static void *trace_get_payload_from_llcsnap(void *link,
 	return (void*)((char*)llc+sizeof(*llc));
 }
 
-void *trace_get_payload_from_pos(void *link, 
+static void *trace_get_payload_from_ppp(void *link, 
 		uint16_t *type, uint32_t *remaining)
 {
 	/* 64 byte capture. */
-	libtrace_pos_t *pos = (libtrace_pos_t*)link;
+	libtrace_ppp_t *ppp = (libtrace_ppp_t*)link;
 
 	if (remaining) {
-		if (*remaining < sizeof(libtrace_pos_t))
+		if (*remaining < sizeof(libtrace_ppp_t))
 			return NULL;
-		*remaining-=sizeof(libtrace_pos_t);
+		*remaining-=sizeof(libtrace_ppp_t);
 	}
 
-	/* This is documented by endace to be an ethertype.  The ethertype
-	 * however is always ntohs(0x0021), which endace in their own tools
-	 * ignore and assume IP, so...
-	 */
-	/*if (type) *type = ntohs(pos->ether_type); */
-	if (type) *type = ntohs(0x0800);
+	if (type) {
+		switch(ntohs(ppp->protocol)) {
+			case 0x0021: *type = 0x0800; break;
+		}
+	}
 
-	return (void*)((char *)pos+sizeof(*pos));
+
+	return (void*)((char *)ppp+sizeof(*ppp));
 }
 
 static void *trace_get_payload_from_pflog(void *link,
@@ -312,59 +319,36 @@ void *trace_get_payload_from_link(void *link, libtrace_linktype_t linktype,
 			
 		case TRACE_TYPE_PFLOG:
 			return trace_get_payload_from_pflog(link,type,remaining);
-		case TRACE_TYPE_POS:
-			return trace_get_payload_from_pos(link,type,remaining);
+		case TRACE_TYPE_PPP:
+			return trace_get_payload_from_ppp(link,type,remaining);
 		case TRACE_TYPE_ATM:
 			l=trace_get_payload_from_atm(link,NULL,remaining);
 			return (l ? trace_get_payload_from_llcsnap(l,
 						type, remaining):NULL);
 		case TRACE_TYPE_DUCK:
 			return NULL; /* duck packets have no payload! */
+		case TRACE_TYPE_METADATA:
+			return NULL; /* The payload is in these packets does
+					not correspond to a genuine link-layer
+					*/
 		default:
 			break;
 	}
-	fprintf(stderr,"Don't understand link layer type %i in trace_get_payload_from_link()\n",
+	fprintf(stderr, "Don't understand link layer type %i in trace_get_payload_from_link()\n",
 		linktype);
 	return NULL;
 }
 
 libtrace_ip_t *trace_get_ip(libtrace_packet_t *packet) 
 {
-	uint16_t type;
-	void *link = trace_get_link(packet);
+	uint16_t ethertype;
 	void *ret;
 
-	if (!link)
-		return NULL;
-	
-	ret=trace_get_payload_from_link(
-			link,
-			trace_get_link_type(packet),
-			&type, NULL);
+	uint32_t remaining = trace_get_capture_length(packet);
 
-	if (!ret)
-		return NULL;
+	ret = trace_get_layer3(packet,&ethertype,&remaining);
 
-	for(;;) {
-		switch(type) {
-			case 0x8100:
-				ret=trace_get_vlan_payload_from_ethernet_payload(ret,&type,NULL);
-				continue;
-			case 0x8847:
-				ret=trace_get_mpls_payload_from_ethernet_payload(ret,&type,NULL);
-
-				if (ret && type == 0x0) {
-					ret=trace_get_payload_from_ethernet(ret,&type,NULL);
-				}
-				continue;
-			default:
-				break;
-		}
-
-		break;
-	}
-
-	if (!ret || type!=0x0800)
+	if (!ret || ethertype!=0x0800)
 		return NULL;
 
 	/* Not an IPv4 packet */
@@ -376,41 +360,14 @@ libtrace_ip_t *trace_get_ip(libtrace_packet_t *packet)
 
 libtrace_ip6_t *trace_get_ip6(libtrace_packet_t *packet) 
 {
-	uint16_t type;
-	void *link=trace_get_link(packet);
+	uint16_t ethertype;
 	void *ret;
-	
-	if (!link)
-		return NULL;
 
-	ret=trace_get_payload_from_link(
-			link,
-			trace_get_link_type(packet),
-			&type,NULL);
+	uint32_t remaining = trace_get_capture_length(packet);
 
-	if (!ret)
-		return NULL;
+	ret = trace_get_layer3(packet,&ethertype,&remaining);
 
-	for(;;) {
-		switch(type) {
-			case 0x8100:
-				ret=trace_get_vlan_payload_from_ethernet_payload(ret,&type,NULL);
-				continue;
-			case 0x8847:
-				ret=trace_get_mpls_payload_from_ethernet_payload(ret,&type,NULL);
-
-				if (ret && type == 0x0) {
-					ret=trace_get_payload_from_ethernet(ret,&type,NULL);
-				}
-				continue;
-			default:
-				break;
-		}
-
-		break;
-	}
-
-	if (!ret || type!=0x86DD)
+	if (!ret || ethertype!=0x86DD)
 		return NULL;
 
 	return (libtrace_ip6_t*)ret;
@@ -484,18 +441,24 @@ void *trace_get_payload_from_ip6(libtrace_ip6_t *ipptr, uint8_t *prot,
 	}
 }
 
-DLLEXPORT void *trace_get_transport(libtrace_packet_t *packet, 
-		uint8_t *proto,
-		uint32_t *remaining
-		) 
+DLLEXPORT void *trace_get_layer3(libtrace_packet_t *packet,
+		uint16_t *ethertype,
+		uint32_t *remaining)
 {
-	void *transport;
-	uint8_t dummy_proto;
-	uint16_t ethertype;
+	void *iphdr;
+	uint16_t dummy_ethertype;
 	void *link;
 	uint32_t dummy_remaining;
 
-	if (!proto) proto=&dummy_proto;
+	/* use l3 cache */
+	if (packet->l3_header)
+	{
+		*ethertype = packet->l3_ethertype;
+		*remaining -= (packet->l3_header - trace_get_link(packet));
+		return packet->l3_header;
+	}
+
+	if (!ethertype) ethertype=&dummy_ethertype;
 
 	if (!remaining) remaining=&dummy_remaining;
 
@@ -506,28 +469,28 @@ DLLEXPORT void *trace_get_transport(libtrace_packet_t *packet,
 	if (!link)
 		return NULL;
 
-	transport = trace_get_payload_from_link(
+	iphdr = trace_get_payload_from_link(
 			link,
 			trace_get_link_type(packet),
-			&ethertype,
+			ethertype,
 			remaining);
 
-	if (!transport)
+	if (!iphdr)
 		return NULL;
 
 	for(;;) {
-		switch(ethertype) {
-		case 0x8100:
-			transport=trace_get_vlan_payload_from_ethernet_payload(
-					  transport,&ethertype,NULL);
+		switch(*ethertype) {
+		case 0x8100: /* VLAN */
+			iphdr=trace_get_vlan_payload_from_ethernet_payload(
+					  iphdr,ethertype,NULL);
 			continue;
-		case 0x8847:
-			transport=trace_get_mpls_payload_from_ethernet_payload(
-					  transport,&ethertype,NULL);
+		case 0x8847: /* MPLS */
+			iphdr=trace_get_mpls_payload_from_ethernet_payload(
+					  iphdr,ethertype,NULL);
 
-			if (transport && ethertype == 0x0) {
-				transport=trace_get_payload_from_ethernet(
-						transport,&ethertype,NULL);
+			if (iphdr && ethertype == 0x0) {
+				iphdr=trace_get_payload_from_ethernet(
+						iphdr,ethertype,NULL);
 			}
 			continue;
 		default:
@@ -537,6 +500,31 @@ DLLEXPORT void *trace_get_transport(libtrace_packet_t *packet,
 		break;
 	}
 
+	/* Store values in the cache for later */
+	packet->l3_ethertype = *ethertype;
+	packet->l3_header = iphdr;
+
+	return iphdr;
+}
+
+DLLEXPORT void *trace_get_transport(libtrace_packet_t *packet, 
+		uint8_t *proto,
+		uint32_t *remaining
+		) 
+{
+	uint8_t dummy_proto;
+	uint16_t ethertype;
+	uint32_t dummy_remaining;
+	void *transport;
+
+	if (!proto) proto=&dummy_proto;
+
+	if (!remaining) remaining=&dummy_remaining;
+
+	*remaining = trace_get_capture_length(packet);
+
+	transport = trace_get_layer3(packet,&ethertype,remaining);
+
 	if (!transport)
 		return NULL;
 
@@ -705,24 +693,45 @@ DLLEXPORT uint16_t trace_get_destination_port(const libtrace_packet_t *packet)
 		return 0;
 }
 
+/* Take a pointer to the start of an IEEE 802.11 MAC frame and return a pointer
+ * to the source MAC address.  
+ * If the frame does not contain a sender address, e.g. ACK frame, return NULL.
+ * If the frame is a 4-address WDS frame, return TA, i.e. addr2.
+ * NB: This function decodes the 802.11 header, so it assumes that there are no
+ * bit-errors. If there are, all bets are off.
+ */
+static
+uint8_t *get_source_mac_from_wifi(void *wifi) {
+	if (wifi == NULL) return NULL;
+	struct libtrace_80211_t *w = (struct libtrace_80211_t *) wifi;
+	
+	/* If the frame is of type CTRL */
+	if (w->type == 0x1) 
+		/* If bit 2 of the subtype field is zero, this indicates that
+		 * there is no transmitter address, i.e. the frame is either an
+		 * ACK or a CTS frame */
+		if ((w->subtype & 0x2) == 0)
+			return NULL;
+
+	/* Always return the address of the transmitter, i.e. address 2 */
+	return (uint8_t *) &w->mac2;
+}
 
 uint8_t *trace_get_source_mac(libtrace_packet_t *packet) {
 	void *link = trace_get_link(packet);
-	libtrace_80211_t *wifi;
         libtrace_ether_t *ethptr = (libtrace_ether_t*)link;
 	if (!link)
 		return NULL;
 	switch (trace_get_link_type(packet)) {
-		case TRACE_TYPE_80211:
-			wifi=(libtrace_80211_t*)link;
-			return (uint8_t*)&wifi->mac2;
 		case TRACE_TYPE_80211_RADIO:
-			wifi=(libtrace_80211_t*)trace_get_payload_from_radiotap(
-					link,NULL,NULL);
-			return (uint8_t*)&wifi->mac1;
+			link = trace_get_payload_from_radiotap(
+					link, NULL, NULL);
+			/* Fall through for 802.11 */
+		case TRACE_TYPE_80211:
+			return get_source_mac_from_wifi(link);
 		case TRACE_TYPE_80211_PRISM:
-			wifi=(libtrace_80211_t*)((char*)link+144);
-			return (uint8_t*)&wifi->mac2;
+			link = ((char*)link+144);
+			return get_source_mac_from_wifi(link);
 		case TRACE_TYPE_ETH:
 			return (uint8_t*)&ethptr->ether_shost;
 		case TRACE_TYPE_POS:
@@ -732,6 +741,7 @@ uint8_t *trace_get_source_mac(libtrace_packet_t *packet) {
 		case TRACE_TYPE_PFLOG:
 		case TRACE_TYPE_ATM:
 		case TRACE_TYPE_DUCK:
+		case TRACE_TYPE_METADATA:
 			return NULL;
 		default:
 			break;
@@ -767,6 +777,7 @@ DLLEXPORT uint8_t *trace_get_destination_mac(libtrace_packet_t *packet) {
 		case TRACE_TYPE_LINUX_SLL:
 		case TRACE_TYPE_PFLOG:
 		case TRACE_TYPE_DUCK:
+		case TRACE_TYPE_METADATA:
 			/* No MAC address */
 			return NULL;
 		default:
@@ -777,10 +788,10 @@ DLLEXPORT uint8_t *trace_get_destination_mac(libtrace_packet_t *packet) {
 	return NULL;
 }
 
-DLLEXPORT struct sockaddr *trace_get_source_address(const libtrace_packet_t *packet, 
-		struct sockaddr *addr)
+DLLEXPORT struct sockaddr *trace_get_source_address(
+		const libtrace_packet_t *packet, struct sockaddr *addr)
 {
-	uint16_t proto;
+	uint16_t ethertype;
 	uint32_t remaining;
 	void *l3;
 	struct ports_t *ports;
@@ -791,41 +802,12 @@ DLLEXPORT struct sockaddr *trace_get_source_address(const libtrace_packet_t *pac
 
 	remaining = trace_get_capture_length(packet);
 
-	l3 = trace_get_payload_from_link(
-			trace_get_link(packet),
-			trace_get_link_type(packet),
-			&proto,
-			&remaining);
-
-	if (!l3)
-		return false;
-
-	for(;;) {
-		switch(proto) {
-			case 0x8100:
-				l3=trace_get_vlan_payload_from_ethernet_payload(
-						l3,&proto,NULL);
-				continue;
-			case 0x8847:
-				l3=trace_get_mpls_payload_from_ethernet_payload(
-						l3,&proto,NULL);
-
-				if (l3 && proto == 0x0) {
-					l3=trace_get_payload_from_ethernet(
-						l3,&proto,NULL);
-				}
-				continue;
-			default:
-				break;
-		}
-
-		break;
-	}
+	l3 = trace_get_layer3(packet,&ethertype,&remaining);
 
 	if (!l3)
 		return NULL;
 
-	switch (proto) {
+	switch (ethertype) {
 		case 0x0800: /* IPv4 */
 		{
 			struct sockaddr_in *addr4=(struct sockaddr_in*)addr;
@@ -848,7 +830,7 @@ DLLEXPORT struct sockaddr *trace_get_source_address(const libtrace_packet_t *pac
 				trace_get_payload_from_ip6(ip6,NULL,&remaining);
 			addr6->sin6_family=AF_INET6;
 			if (ports && remaining>=sizeof(*ports))
-				addr6->sin6_port=ports->dst;
+				addr6->sin6_port=ports->src;
 			else
 				addr6->sin6_port=0;
 			addr6->sin6_flowinfo=0;
@@ -860,12 +842,13 @@ DLLEXPORT struct sockaddr *trace_get_source_address(const libtrace_packet_t *pac
 	}
 }
 
-DLLEXPORT struct sockaddr *trace_get_destination_address(const libtrace_packet_t *packet, 
-		struct sockaddr *addr)
+DLLEXPORT struct sockaddr *trace_get_destination_address(
+		const libtrace_packet_t *packet, struct sockaddr *addr)
 {
-	uint16_t proto;
+	uint16_t ethertype;
 	uint32_t remaining;
-	void *transport;
+	void *l3;
+	struct ports_t *ports;
 	static struct sockaddr_storage dummy;
 
 	if (!addr)
@@ -873,53 +856,37 @@ DLLEXPORT struct sockaddr *trace_get_destination_address(const libtrace_packet_t
 
 	remaining = trace_get_capture_length(packet);
 
-	transport = trace_get_payload_from_link(
-			trace_get_link(packet),
-			trace_get_link_type(packet),
-			&proto,
-			&remaining);
+	l3 = trace_get_layer3(packet,&ethertype,&remaining);
 
-	if (!transport)
-		return false;
-	for(;;) {
-		switch(proto) {
-			case 0x8100:
-				transport=trace_get_vlan_payload_from_ethernet_payload(transport,&proto,NULL);
-				continue;
-			case 0x8847:
-				transport=trace_get_mpls_payload_from_ethernet_payload(transport,&proto,NULL);
+	if (!l3)
+		return NULL;
 
-				if (transport && proto == 0x0) {
-					transport=trace_get_payload_from_ethernet(
-							transport,&proto,NULL);
-				}
-				continue;
-			default:
-				break;
-		}
-
-		break;
-	}
-
-	if (!transport)
-		return false;
-
-	switch (proto) {
+	switch (ethertype) {
 		case 0x0800: /* IPv4 */
 		{
 			struct sockaddr_in *addr4=(struct sockaddr_in*)addr;
-			libtrace_ip_t *ip = (libtrace_ip_t*)transport;
+			libtrace_ip_t *ip = (libtrace_ip_t*)l3;
+			ports = (struct ports_t*)
+				trace_get_payload_from_ip(ip,NULL,&remaining);
 			addr4->sin_family=AF_INET;
-			addr4->sin_port=0;
+			if (ports && remaining>=sizeof(*ports))
+				addr4->sin_port=ports->dst;
+			else
+				addr4->sin_port=0;
 			addr4->sin_addr=ip->ip_dst;
 			return addr;
 		}
 		case 0x86DD: /* IPv6 */
 		{
 			struct sockaddr_in6 *addr6=(struct sockaddr_in6*)addr;
-			libtrace_ip6_t *ip6 = (libtrace_ip6_t*)transport;
+			libtrace_ip6_t *ip6 = (libtrace_ip6_t*)l3;
+			ports = (struct ports_t*)
+				trace_get_payload_from_ip6(ip6,NULL,&remaining);
 			addr6->sin6_family=AF_INET6;
-			addr6->sin6_port=0;
+			if (ports && remaining>=sizeof(*ports))
+				addr6->sin6_port=ports->dst;
+			else
+				addr6->sin6_port=0;
 			addr6->sin6_flowinfo=0;
 			addr6->sin6_addr=ip6->ip_dst;
 			return addr;
