@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: trace.c 1133 2007-02-07 03:46:43Z spa1 $
+ * $Id: trace.c 1263 2007-09-05 04:34:10Z perry $
  *
  */
 
@@ -221,7 +221,9 @@ static void trace_init(void)
 	if (!formats_list) {
 		duck_constructor();
 		erf_constructor();
+		tsh_constructor();
 		legacy_constructor();
+		atmhdr_constructor();
 #ifdef HAVE_NETPACKET_PACKET_H
 		linuxnative_constructor();
 #endif
@@ -233,7 +235,6 @@ static void trace_init(void)
 #endif
 		pcapfile_constructor();
 		rt_constructor();
-		wag_constructor();
 #ifdef HAVE_DAG
 		dag_constructor();
 #endif
@@ -275,10 +276,6 @@ DLLEXPORT void trace_help(void) {
  *  pcap:-
  *  rtclient:hostname
  *  rtclient:hostname:port
- *  wag:-
- *  wag:/path/to/wag/file
- *  wag:/path/to/wag/file.gz
- *  wag:/path/to/wag/socket
  *
  * If an error occured when attempting to open a trace, NULL is returned
  * and an error is output to stdout.
@@ -302,12 +299,6 @@ DLLEXPORT libtrace_t *trace_create(const char *uri) {
 	libtrace->err.err_num = TRACE_ERR_NOERROR;
 	libtrace->format=NULL;
         
-        /* parse the URI to determine what sort of event we are dealing with */
-	if ((uridata = trace_parse_uri(uri, &scan)) == 0) {
-		trace_set_err(libtrace,TRACE_ERR_BAD_FORMAT,"Bad uri format (%s)",uri);
-		return libtrace;
-	}
-	
 	libtrace->event.tdelta = 0.0;
 	libtrace->event.packet = NULL;
 	libtrace->event.psize = 0;
@@ -316,6 +307,12 @@ DLLEXPORT libtrace_t *trace_create(const char *uri) {
 	libtrace->snaplen = 0;
 	libtrace->started=false;
 
+        /* parse the URI to determine what sort of event we are dealing with */
+	if ((uridata = trace_parse_uri(uri, &scan)) == 0) {
+		trace_set_err(libtrace,TRACE_ERR_BAD_FORMAT,"Bad uri format (%s)",uri);
+		return libtrace;
+	}
+	
 	for (tmp=formats_list;tmp;tmp=tmp->next) {
 		if (strlen(scan) == strlen(tmp->name) &&
 				strncasecmp(scan, tmp->name, strlen(scan)) == 0
@@ -558,13 +555,21 @@ DLLEXPORT int trace_config(libtrace_t *libtrace,
 						"Promisc mode is not supported by this format module");
 			}
 			return -1;
-		case TRACE_META_FREQ:
+		case TRACE_OPTION_META_FREQ:
 			if (!trace_is_err(libtrace)) {
 				trace_set_err(libtrace, 
 						TRACE_ERR_OPTION_UNAVAIL,
 						"This format does not support meta-data gathering");
 			}
 			return -1;
+		case TRACE_OPTION_EVENT_REALTIME:
+			if (!trace_is_err(libtrace)) {
+				trace_set_err(libtrace, 
+						TRACE_ERR_OPTION_UNAVAIL,
+						"This format does not support meta-data gathering");
+			}
+			return -1;
+			
 	}
 	if (!trace_is_err(libtrace)) {
 		trace_set_err(libtrace,TRACE_ERR_UNKNOWN_OPTION,
@@ -620,7 +625,8 @@ DLLEXPORT void trace_destroy_dead(libtrace_t *libtrace) {
 DLLEXPORT void trace_destroy_output(libtrace_out_t *libtrace) 
 {
 	assert(libtrace);
-	libtrace->format->fin_output(libtrace);
+	if (libtrace->format)
+		libtrace->format->fin_output(libtrace);
 	free(libtrace->uridata);
 	free(libtrace);
 }
@@ -629,7 +635,9 @@ DLLEXPORT libtrace_packet_t *trace_create_packet(void)
 {
 	libtrace_packet_t *packet = 
 		(libtrace_packet_t*)calloc((size_t)1,sizeof(libtrace_packet_t));
+
 	packet->buf_control=TRACE_CTRL_PACKET;
+	packet->capture_length=-1;
 	return packet;
 }
 
@@ -645,6 +653,11 @@ DLLEXPORT libtrace_packet_t *trace_copy_packet(const libtrace_packet_t *packet)
 		((char*)dest->buffer+trace_get_framing_length(packet));
 	dest->type=packet->type;
 	dest->buf_control=TRACE_CTRL_PACKET;
+	/* Reset the cache - better to recalculate than try to convert
+	 * the values over to the new packet */
+	dest->capture_length = -1;
+	dest->l3_header = NULL;
+	dest->l3_ethertype = 0;
 	memcpy(dest->header,packet->header,trace_get_framing_length(packet));
 	memcpy(dest->payload,packet->payload,trace_get_capture_length(packet));
 
@@ -685,6 +698,18 @@ DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 	 * structure */
 	packet->trace = libtrace;
 
+	/* Finalise the packet, freeing any resources the format module
+	 * may have allocated it
+	 */
+	if (libtrace->format->fin_packet) {
+		libtrace->format->fin_packet(packet);
+	}
+
+	/* Clear the packet cache */
+	packet->capture_length = -1;
+	packet->l3_header = NULL;
+	packet->l3_ethertype = 0;
+
 	if (libtrace->format->read_packet) {
 		do {
 			size_t ret;
@@ -835,12 +860,17 @@ DLLEXPORT double trace_get_seconds(const libtrace_packet_t *packet) {
 	return seconds;
 }
 
-DLLEXPORT size_t trace_get_capture_length(const libtrace_packet_t *packet) 
+DLLEXPORT size_t trace_get_capture_length(libtrace_packet_t *packet) 
 {
-	if (packet->trace->format->get_capture_length) {
-		return packet->trace->format->get_capture_length(packet);
+	/* Cache the capture length */
+	if (packet->capture_length == -1) {
+		if (!packet->trace->format->get_capture_length)
+			return ~0U;
+		packet->capture_length = 
+			packet->trace->format->get_capture_length(packet);
 	}
-	return ~0U;
+
+	return packet->capture_length;
 }
 	
 /* Get the size of the packet as it was seen on the wire.
@@ -1255,7 +1285,7 @@ DLLEXPORT const char * trace_parse_uri(const char *uri, char **format) {
                 return 0;
         }
 
-        if ((uridata - uri) > URI_PROTO_LINE) {
+        if ((unsigned)(uridata - uri) > URI_PROTO_LINE) {
                 /* badly formed URI - uri type is too long */
                 return 0;
         }
@@ -1328,7 +1358,9 @@ DLLEXPORT void trace_perror_output(libtrace_out_t *trace,const char *msg,...)
 	va_end(va);
 	if(trace->err.err_num) {
 		fprintf(stderr,"%s(%s): %s\n",
-				buf,trace->uridata,trace->err.problem);
+				buf,
+				trace->uridata?trace->uridata:"no uri",
+				trace->err.problem);
 	} else {
 		fprintf(stderr,"%s(%s): No error\n",buf,trace->uridata);
 	}
@@ -1419,14 +1451,13 @@ DLLEXPORT int trace_seek_timeval(libtrace_t *trace, struct timeval tv)
 
 DLLEXPORT char *trace_ether_ntoa(const uint8_t *addr, char *buf)
 {
-	char *buf2 = buf;
-	char staticbuf[18]={0,};
-	if (!buf2)
-		buf2=staticbuf;
-	snprintf(buf2,(size_t)18,"%02x:%02x:%02x:%02x:%02x:%02x",
+	static char staticbuf[18]={0,};
+	if (!buf)
+		buf=staticbuf;
+	snprintf(buf,(size_t)18,"%02x:%02x:%02x:%02x:%02x:%02x",
 			addr[0],addr[1],addr[2],
 			addr[3],addr[4],addr[5]);
-	return buf2;
+	return buf;
 }
 
 DLLEXPORT uint8_t *trace_ether_aton(const char *buf, uint8_t *addr)
@@ -1488,5 +1519,5 @@ void trace_construct_packet(libtrace_packet_t *packet,
 	packet->payload=(void*)((char*)packet->buffer+sizeof(hdr));
 	memcpy(packet->header,&hdr,sizeof(hdr));
 	memcpy(packet->payload,data,(size_t)len);
-	packet->type=pcap_dlt_to_rt(libtrace_to_pcap_dlt(linktype));
+	packet->type=pcap_linktype_to_rt(libtrace_to_pcap_linktype(linktype));
 }
