@@ -37,23 +37,26 @@ static int usage(char *argv0)
 	"-e --endtime=time	End at time\n"
 	"-m --maxfiles=n	Create a maximum of n trace files\n"
 	"-H --libtrace-help	Print libtrace runtime documentation\n"
+	"-S --snaplen		Snap packets at the specified length\n"
 	,argv0);
 	exit(1);
 }
 
-int done=0;
+volatile int done=0;
 
-void cleanup_signal(int sig)
+static void cleanup_signal(int sig)
 {
 	done=1;
 }
 
 int main(int argc, char *argv[])
 {
+	/* All these variables are getting silly */
 	struct libtrace_filter_t *filter=NULL;
 	struct libtrace_out_t *output = NULL;
 	struct libtrace_t *input;
 	struct libtrace_packet_t *packet = trace_create_packet();
+	struct sigaction sigact;
 	uint64_t count=UINT64_MAX;
 	uint64_t bytes=UINT64_MAX;
 	uint64_t starttime=0;
@@ -65,6 +68,7 @@ int main(int argc, char *argv[])
 	uint64_t totbyteslast=0;
 	uint64_t maxfiles = UINT64_MAX;
 	uint64_t filescreated = 0;
+	uint16_t snaplen = 0;
 	
 	if (argc<2) {
 		usage(argv[0]);
@@ -83,10 +87,11 @@ int main(int argc, char *argv[])
 			{ "interval",	   1, 0, 'i' },
 			{ "libtrace-help", 0, 0, 'H' },
 			{ "maxfiles", 	   1, 0, 'm' },
+			{ "snaplen",	   1, 0, 'S' },
 			{ NULL, 	   0, 0, 0   },
 		};
 
-		int c=getopt_long(argc, argv, "f:c:b:s:e:i:m:H",
+		int c=getopt_long(argc, argv, "f:c:b:s:e:i:m:S:H",
 				long_options, &option_index);
 
 		if (c==-1)
@@ -107,6 +112,8 @@ int main(int argc, char *argv[])
 				  break;
 			case 'm': maxfiles=atoi(optarg);
 				  break;
+			case 'S': snaplen=atoi(optarg);
+				  break;
 			case 'H':
 				  trace_help();
 				  exit(1);
@@ -122,15 +129,22 @@ int main(int argc, char *argv[])
 		usage(argv[0]);
 	}
 
+	sigact.sa_handler = cleanup_signal;
+	sigemptyset(&sigact.sa_mask);
+	sigact.sa_flags = SA_RESTART;
+
+	sigaction(SIGINT, &sigact, NULL);
+	sigaction(SIGTERM, &sigact, NULL);
+
 	output=NULL;
 	input=trace_create(argv[optind]);
 	if (trace_is_err(input)) {
-		trace_perror(input,"");
+		trace_perror(input,"%s",argv[optind]);
 		return 1;
 	}
 
 	if (trace_start(input)==-1) {
-		trace_perror(input,"");
+		trace_perror(input,"%s",argv[optind]);
 		return 1;
 	}
 
@@ -142,6 +156,10 @@ int main(int argc, char *argv[])
 		if (trace_read_packet(input,packet)<1) {
 			break;
 		}
+
+		if (snaplen>0) {
+			trace_set_capture_length(packet,snaplen);
+		}
 		
 		if (filter && !trace_apply_filter(filter,packet)) {
 			continue;
@@ -183,21 +201,31 @@ int main(int argc, char *argv[])
 				break;
 			}
 			buffer=strdup(argv[optind+1]);
-			if (interval!=UINT64_MAX) {
+			if (interval!=UINT64_MAX && maxfiles>1) {
 				buffer=strdupcat(buffer,"-");
 				buffer=strdupcati(buffer,(uint64_t)firsttime);
 			}
-			if (count!=UINT64_MAX) {
+			if (count!=UINT64_MAX && maxfiles>1) {
 				buffer=strdupcat(buffer,"-");
 				buffer=strdupcati(buffer,(uint64_t)pktcount);
 			}
-			if (bytes!=UINT64_MAX) {
+			if (bytes!=UINT64_MAX && maxfiles>1) {
 				static int filenum=0;
 				buffer=strdupcat(buffer,"-");
 				buffer=strdupcati(buffer,(uint64_t)++filenum);
 			}
 			output=trace_create_output(buffer);
+			if (trace_is_err_output(output)) {
+				trace_perror_output(output,"%s",buffer);
+				free(buffer);
+				break;
+			}
 			trace_start_output(output);
+			if (trace_is_err_output(output)) {
+				trace_perror_output(output,"%s",buffer);
+				free(buffer);
+				break;
+			}
 			free(buffer);
 			filescreated ++;
 		}
